-- Pakoskyfrog 2013/02/23 15:11:07

------------------------
--  This is an example of simple collisions
-- in a complete minigame where you can play alone : a circular Pong !

------------------------
--  Init
local platform = {}
local ball = {}
local bestTime = 0
local currentTime = 0
local showGeometrics = false
ball.toc = 0

local ox, oy = love.graphics.getWidth()/2, love.graphics.getHeight()/2
platform.radius = math.min(ox, oy) - 25

------------------------
--  Callbacks
function love.load()
    initGame()
end
function initGame()
    currentTime = 0
    platform.length = 50
    -- platform.angle  = math.pi/2 -- start at the bottom position
    platform.angle  = math.pi*2 * math.random() -- start anywhere
    platform.omega = 0 -- platform speed
    platform.acc   = 2.2 -- acceleration factor
    
    ball.dir = {math.random()-0.5, math.random()-0.5}
    ball.pos = {ox, oy}
    ball.speed = 75
    normalizeDirection(ball.dir, ball.speed/10)
end

function love.keypressed(key)
    if key == 'left' then
        platform.goingLeft = true
    elseif key == 'right' then
        if not (platform.goingLeft) then
            platform.goingRight = true
        end
    end
    
    if key == 'escape' then
        initGame()
    end
    if key == 'g' then
        showGeometrics = not showGeometrics
    end
end
function love.keyreleased(key)
    if key == 'left' then
        platform.goingLeft = false
    elseif key == 'right' then
        platform.goingRight = false
    end
end

function love.update(dt)
    -- platform moving ?
    if platform.goingLeft then
        if platform.omega > 0 then
            platform.omega = platform.omega + platform.acc*dt
        else
            platform.omega = platform.omega + platform.acc*dt*2
        end
    elseif platform.goingRight then
        if platform.omega < 0 then
            platform.omega = platform.omega - platform.acc*dt
        else
            platform.omega = platform.omega - platform.acc*dt*2
        end
    end
    platform.angle = platform.angle + platform.omega*dt
    
    -- ball movement
    ball.pos[1] = ball.pos[1] + ball.dir[1]
    ball.pos[2] = ball.pos[2] + ball.dir[2]

    -- collision ?
    -- avoiding multi simultaneous collisions
    if ball.toc > 0.1 and isBallCollinding() then
        bounce()
    end
    
    -- acceleration over time
    ball.speed = ball.speed + dt*2
    normalizeDirection(ball.dir, ball.speed*dt)
    
    -- time recording
    currentTime = currentTime + dt
    ball.toc = ball.toc + dt
    
    -- Lost ?
    if math.sqrt((ox-ball.pos[1])^2 + (oy-ball.pos[2])^2) > 50 + platform.radius then
        -- yep !!
        bestTime = math.max(bestTime, currentTime)
        initGame()
    end
end

function love.draw()
    love.graphics.setColor(255,255,255)
    love.graphics.print('Press "g" to toggle ON/OFF geometrics', 10,10)
    love.graphics.print('Press "escape" to reinit the game', 10,30)
    
    if showGeometrics then
        local theta = math.atan2(platform.length,(platform.radius)) % (2*math.pi) -- half openning angle
        local alpha = math.atan2(ball.pos[2]-oy, ball.pos[1]-ox) % (2*math.pi)
        local beta  = math.atan2(ball.dir[2], ball.dir[1])
        
        local at, aT = (platform.angle-theta) % (2*math.pi), (platform.angle+theta) % (2*math.pi)
        if aT < at then aT = aT + 2*math.pi end
        local open = ((at < alpha) and (aT > alpha)) or ((at-2*math.pi < alpha) and (aT-2*math.pi > alpha))
        
        -- ball direction
        love.graphics.setColor(200,200,200)
        love.graphics.setLineWidth(1)
        love.graphics.line(ox,oy, ox+100*math.cos(alpha),oy+100*math.sin(alpha))
        
        -- ball speed
        love.graphics.setColor(200,200,100)
        love.graphics.setLineWidth(1)
        love.graphics.line(ball.pos[1],ball.pos[2], ball.pos[1]+ball.dir[1]*10,ball.pos[2]+ball.dir[2]*10)
        
        if open then love.graphics.setColor(0,127,0)
        else love.graphics.setColor(127,0,0) end
        love.graphics.setLineWidth(1)
        -- entrance arc
        love.graphics.arc('fill', ox, oy, 200, at, aT, 16)
        
        do
            love.graphics.setColor(50,50,255)
            love.graphics.setLineWidth(1)
            local beta  = math.atan2(ball.dir[2], ball.dir[1]) % (2*math.pi)
            local reflex = (2*platform.angle - (beta+math.pi)) % (math.pi*2)
            local cx = ox + platform.radius * math.cos(platform.angle)
            local cy = oy + platform.radius * math.sin(platform.angle)
            
            -- bounce line
            love.graphics.line(cx,cy, cx+70*math.cos(reflex),cy+70*math.sin(reflex))
        end
    end
    love.graphics.setColor(25,25,25)
    love.graphics.setLineWidth(3)
    love.graphics.circle('line', ox, oy, platform.radius, 128)
    
    drawPlatform()
    drawBall()
    drawInfos()
end
function drawPlatform()
    love.graphics.setColor(255,127,127)
    love.graphics.setLineWidth(15)
    local x1,y1, x2,y2
    local cx = ox + platform.radius * math.cos(platform.angle)
    local cy = oy + platform.radius * math.sin(platform.angle)
    
    -- calculate platform extremities from the center
    x1 = cx - platform.length * math.sin(platform.angle)
    y1 = cy + platform.length * math.cos(platform.angle)
    x2 = cx + platform.length * math.sin(platform.angle)
    y2 = cy - platform.length * math.cos(platform.angle)
    
    love.graphics.line(x1,y1, x2,y2)
end
function drawBall()
    love.graphics.setColor(127,255,127)
    love.graphics.setPointSize(15)
    love.graphics.point(ball.pos[1], ball.pos[2])
end
function drawInfos()
    --------------------
    --  this will print the current time and the best time
    love.graphics.setColor(127,127,255)
    local c = math.floor(currentTime*100)/100
    local b = math.floor(bestTime*100)/100
    local s = math.floor(ball.speed*100)/100
    love.graphics.print('Best Time : '..b, 10, 50)
    love.graphics.print('Ball speed = '..s, 10, 90)
    if currentTime > bestTime then love.graphics.setColor(127,255,127) end
    love.graphics.print('Timer = '..c, 10, 70)
end

------------------------
--  functions

function normalizeDirection(vector, speed)
    --------------------
    --  this will make vector to have the same direction it has but with a length = speed
    -- Warning : it modifies the vector itself
    local l = math.sqrt(vector[1]^2 + vector[2]^2)
    vector[1] = vector[1]/l*speed
    vector[2] = vector[2]/l*speed
end

function distToAffine(a,b, M)
    --------------------
    --  this calculates the distance from an affine func ax+b to a point M
    return math.abs(a*M[1] - M[2] + b) / math.sqrt(a*a+1)
end
function insideSegment(A,B, M)
    ------------------------
    --  this returns true if M is between A and B
    -- each points are {num, num}
    
    -- was my first method to detect collisions, not used anymore
    
    -- half thicknesses of the segment
    local e = 10/2
    local d = math.sqrt((B[1]-A[1])^2 + (B[2]-A[2])^2)/2
    
    -- affines coefs :
    -- (AB) :
    local a = (B[2]-A[2])/(B[1]-A[1])
    local b = A[2]-a*A[1]
    
    -- (AB) mediatrix :
    local I = {0.5*(A[1]+B[1]), 0.5*(A[2]+B[2])} -- med point AI=IB
    local ma = -1/a
    local mb = I[2]-ma*I[1]
    
    -- slopes to M distances :
    local s  = distToAffine(a,b,   M)
    local ms = distToAffine(ma,mb, M)
    
    return (ms < d) and (s < e)
end

function isBallCollinding()
    --------------------
    --  this return true if the ball reach the platform
    local theta = math.atan2(platform.length,(platform.radius)) % (2*math.pi) -- half openning angle
    local alpha = math.atan2(ball.pos[2]-oy, ball.pos[1]-ox) % (2*math.pi)    -- arg of ball
    
    -- lower and upper bound
    local at, aT = (platform.angle-theta) % (2*math.pi), (platform.angle+theta) % (2*math.pi)
    if aT < at then aT = aT + 2*math.pi end
    
    -- angle condition
    local isInAngle = ((at < alpha) and (aT > alpha)) or ((at-2*math.pi < alpha) and (aT-2*math.pi > alpha))
    
    -- dist condition
    local distFromCenter = math.sqrt((ox-ball.pos[1])^2 + (oy-ball.pos[2])^2)
    local isOnPlatform = (distFromCenter > platform.radius - 5) and (distFromCenter < platform.radius + 5)
    
    local collinding = isInAngle and isOnPlatform
    return collinding
end


function bounce()
    --------------------
    --  this happens when you chatch the ball !
    local beta  = math.atan2(ball.dir[2], ball.dir[1])
    local reflex = (2*platform.angle - (beta+math.pi)) % (math.pi*2)
    ball.dir = {math.cos(reflex), math.sin(reflex)}
    normalizeDirection(ball.dir, ball.speed)
    ball.toc = 0 -- time since collinding
end
